package net.pingex.dcf.permissions;

import net.pingex.dcf.core.Configuration;
import sx.blah.discord.handle.obj.IMessage;
import java.util.function.Predicate;

/**
 * Default behavior when a permissions provider doesn't return `true` or `false`
 */
public enum DefaultPermission implements Predicate<IMessage>
{
	/**
	 * Everyone is allowed to run the command.
	 */
	EVERYONE(iMessage -> true),

	/**
	 * Only the guild owner is allowed to run the command.
	 */
	GUILD_OWNER(iMessage -> iMessage.getAuthor().getID().equals(iMessage.getGuild().getOwnerID())),

	/**
	 * Only the bot owner is allowed to run the command.
	 */
	BOT_OWNER(iMessage -> iMessage.getAuthor().getID().equals(Configuration.BOT_OWNER)),

	/**
	 * Guild Owner x Bot Owner
	 */
	ANY_OWNER(iMessage -> GUILD_OWNER.test(iMessage) || BOT_OWNER.test(iMessage)),

	/**
	 * Nobody is allowed to run the command
	 */
	NONE(iMessage -> false);

	DefaultPermission(Predicate<IMessage> predicate)
	{
		this.predicate = predicate;
	}

	private Predicate<IMessage> predicate;

	@Override
	public boolean test(IMessage o)
	{
		return predicate.test(o);
	}
}
