package net.pingex.dcf.permissions;

import net.pingex.dcf.commands.Command;
import sx.blah.discord.handle.obj.IGuild;
import sx.blah.discord.handle.obj.IRole;
import sx.blah.discord.handle.obj.IUser;

/**
 * Provider of permissions, ie. the proxy between {@code PermissionsHandler} and a data storage.
 *
 * The validator will successively test the following until something other than `null` is returned.
 * - Individual user permission using {@code validateUser()}
 * - The user's role using {@code validateGroup()}
 * - Default behavior
 */
public interface ICommandPermissionsProvider
{
	/**
	 * Validate individual user permission.
	 * @param guild Guild where command is executed. `null` = globally
	 * @param user Target user
	 * @param command Command to test
	 * @return `true` if the user is able to run the command, `false` if he isn't able to, `null` if there is no answer.
	 */
	Boolean validateUser(IGuild guild, IUser user, Command command);

	/**
	 * Validate a role's ability to run a command.
	 * @param role Target role
	 * @param command Command to test
	 * @return `true` if the role is able to run the command, `false` if he isn't able to, `null` if there is no answer.
	 */
	Boolean validateGroup(IRole role, Command command);
}
