package net.pingex.dcf.permissions;

import net.pingex.dcf.commands.Command;
import net.pingex.dcf.commands.CommandRegistry;
import net.pingex.dcf.commands.IWithCommands;
import net.pingex.dcf.util.DiscordInteractionsUtil;
import sx.blah.discord.handle.impl.events.MessageReceivedEvent;
import sx.blah.discord.handle.obj.IUser;

import java.util.*;
import java.util.regex.Pattern;

/**
 * Commands for the permissions package.
 */
public class PermissionsCommands implements IWithCommands
{
	@Override
	public Set<Command> getCommands()
	{
		return new HashSet<>(Arrays.asList(
			IsAllowedCommand
		));
	}

	private static final Command IsAllowedCommand =
			new Command.Builder("canRun")
			.description("Tells whether target user is allowed to run the command.")
			.enabled(true)
			.usage("Command <User#Disc|ID> [verbose]")
			.defaultPermission(DefaultPermission.BOT_OWNER)
			.build(PermissionsCommands::isAllowedImpl);

	private static Pattern usernamePattern = Pattern.compile("^.*#\\d{4}$");
	private static Pattern idPattern = Pattern.compile("^\\d{18}$");

	private static void isAllowedImpl(MessageReceivedEvent event, List<String> arguments)
	{
		// Parameters
		Command target = null;
		IUser userToLookup = null;
		boolean verbose = false;

		// Argchk size
		if(arguments.size() != 2 && arguments.size() != 3)
		{
			DiscordInteractionsUtil.sendMessage(event.getMessage().getChannel(), "Invalid arguments.");
			return;
		}

		// Argchk#1 - valid command
		Optional<Command> uncheckedTarget = CommandRegistry.getCommandOrAliasByName(arguments.get(0));
		if(!uncheckedTarget.isPresent()) // Command existence
		{
			DiscordInteractionsUtil.sendMessage(event.getMessage().getChannel(), "Target command not found.");
			return;
		}
		else target = uncheckedTarget.get();

		// Argchk#2 - uid OR username#1234 OR "me"
		if(idPattern.matcher(arguments.get(1)).matches()) // uid
		{
			userToLookup = event.getClient().getUserByID(arguments.get(1));
		}
		else if(usernamePattern.matcher(arguments.get(1)).matches()) // username#1234
		{
			for(IUser iUser : event.getClient().getUsers()) // Greedy
				if(arguments.get(1).equalsIgnoreCase(iUser.getName() + "#" + iUser.getDiscriminator()))
				{
					userToLookup = iUser;
					break; // Limit greediness
				}
		}
		else if(arguments.get(1).equalsIgnoreCase("me")) // me
		{
			userToLookup = event.getMessage().getAuthor();
		}
		else // ?
		{
			DiscordInteractionsUtil.sendMessage(event.getMessage().getChannel(), "Invalid arguments.");
			return;
		}
		if(userToLookup == null)
		{
			DiscordInteractionsUtil.sendMessage(event.getMessage().getChannel(), "User not found.");
			return;
		}

		// Argchk#3 - verbosity
		if(arguments.size() == 3) verbose = true;

		// ========================================

		DiscordInteractionsUtil.sendMessage(event.getMessage().getChannel(), "STUB: Checking perms for command " + target.getName() + " and user " + userToLookup.getName());
	}
}
